@@ -1,6 +1,6 @@
 #!/bin/bash -ue
 # Copyright (C) 2013 Percona Inc
-# Copyright (C) 2017-2020 MariaDB
+# Copyright (C) 2017-2019 MariaDB
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -182,60 +182,38 @@ get_transfer()
     fi
 
     if [[ $tfmt == 'nc' ]];then
-	wsrep_check_programs nc
-
+        wsrep_check_programs nc
         wsrep_log_info "Using netcat as streamer"
         if [[ "$WSREP_SST_OPT_ROLE"  == "joiner" ]];then
             if nc -h 2>&1 | grep -q ncat;then 
                 # Ncat
                 tcmd="nc -l ${TSST_PORT}"
-            elif nc -h 2>&1 | grep -qw -- '-d\>';then
+            elif nc -h 2>&1 | grep -q -- '-d\>';then
                 # Debian netcat
-                if [ $WSREP_SST_OPT_HOST_IPv6 -eq 1 ];then
-                    # When host is not explicitly specified (when only the port
-                    # is specified) netcat can only bind to an IPv4 address if
-                    # the "-6" option is not explicitly specified:
-                    tcmd="nc -dl -6 ${TSST_PORT}"
-                else
-                    tcmd="nc -dl ${TSST_PORT}"
-                fi
+                tcmd="nc -dl ${TSST_PORT}"
             else
                 # traditional netcat
                 tcmd="nc -l -p ${TSST_PORT}"
             fi
         else
-            # Check to see if netcat supports the '-N' flag.
-            # -N Shutdown the network socket after EOF on stdin
-            # If it supports the '-N' flag, then we need to use the '-N'
-            # flag, otherwise the transfer will stay open after the file
-            # transfer and cause the command to timeout.
-            # Older versions of netcat did not need this flag and will
-            # return an error if the flag is used.
-            #
-            tcmd_extra=""
-            if nc -h 2>&1 | grep -qw -- -N; then
-                tcmd_extra+="-N"
-		wsrep_log_info "Using nc -N"
-            fi
-
-            # netcat doesn't understand [] around IPv6 address
             if nc -h 2>&1 | grep -q ncat;then
                 # Ncat
-                wsrep_log_info "Using Ncat as streamer"
-                tcmd="nc ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
-            elif nc -h 2>&1 | grep -qw -- '-d\>';then
+                tcmd="nc ${REMOTEIP} ${TSST_PORT}"
+            elif nc -h 2>&1 | grep -q -- '-d\>';then
                 # Debian netcat
-                wsrep_log_info "Using Debian netcat as streamer"
-                tcmd="nc ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
+                if nc -h 2>&1 | grep -q -- '-N\>';then
+                   tcmd="nc -N ${REMOTEIP} ${TSST_PORT}"
+                else
+                   tcmd="nc ${REMOTEIP} ${TSST_PORT}"
+                fi
             else
                 # traditional netcat
-                wsrep_log_info "Using traditional netcat as streamer"
-                tcmd="nc -q0 ${tcmd_extra} ${WSREP_SST_OPT_HOST_UNESCAPED} ${TSST_PORT}"
+                tcmd="nc -q0 ${REMOTEIP} ${TSST_PORT}"
             fi
         fi
     else
         tfmt='socat'
-	wsrep_check_programs socat
+        wsrep_check_programs socat
         wsrep_log_info "Using socat as streamer"
 
         if [[ $encrypt -eq 2 || $encrypt -eq 3 ]] && ! socat -V | grep -q "WITH_OPENSSL 1";then
@@ -384,7 +362,7 @@ read_cnf()
     iopts=$(parse_cnf sst inno-backup-opts "")
     iapts=$(parse_cnf sst inno-apply-opts "")
     impts=$(parse_cnf sst inno-move-opts "")
-    stimeout=$(parse_cnf sst sst-initial-timeout 300)
+    stimeout=$(parse_cnf sst sst-initial-timeout 100)
     ssyslog=$(parse_cnf sst sst-syslog 0)
     ssystag=$(parse_cnf mysqld_safe syslog-tag "${SST_SYSLOG_TAG:-}")
     ssystag+="-"
@@ -548,11 +526,25 @@ kill_xtrabackup()
 
 setup_ports()
 {
-    SST_PORT=${WSREP_SST_OPT_ADDR_PORT}
     if [[ "$WSREP_SST_OPT_ROLE"  == "donor" ]];then
-        REMOTEIP=${WSREP_SST_OPT_HOST}
-        lsn=${WSREP_SST_OPT_LSN}
-        sst_ver=${WSREP_SST_OPT_SST_VER}
+        if [ "${WSREP_SST_OPT_ADDR#\[}" != "$WSREP_SST_OPT_ADDR" ]; then
+            remain=$(echo $WSREP_SST_OPT_ADDR | awk -F '\\][:/]' '{ print $2 }')
+            REMOTEIP=$(echo $WSREP_SST_OPT_ADDR | awk -F '\\]:' '{ print $1 }')"]"
+            SST_PORT=$(echo $remain | awk -F '[:/]' '{ print $1 }')
+            lsn=$(echo $remain | awk -F '[:/]' '{ print $3 }')
+            sst_ver=$(echo $remain | awk -F '[:/]' '{ print $4 }')
+        else
+            SST_PORT=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $2 }')
+            REMOTEIP=$(echo $WSREP_SST_OPT_ADDR | awk -F ':' '{ print $1 }')
+            lsn=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $4 }')
+            sst_ver=$(echo $WSREP_SST_OPT_ADDR | awk -F '[:/]' '{ print $5 }')
+        fi
+    else
+        if [ "${WSREP_SST_OPT_ADDR#\[}" != "$WSREP_SST_OPT_ADDR" ]; then
+            SST_PORT=$(echo ${WSREP_SST_OPT_ADDR} | awk -F '\\]:' '{ print $2 }')
+        else
+            SST_PORT=$(echo ${WSREP_SST_OPT_ADDR} | awk -F ':' '{ print $2 }')
+        fi
     fi
 }
 
@@ -631,8 +623,7 @@ recv_joiner()
     popd 1>/dev/null 
 
     if [[ ${RC[0]} -eq 124 ]];then 
-        wsrep_log_error "Possible timeout in receiving first data from "
-	                "donor in gtid stage: exit codes: ${RC[@]}"
+        wsrep_log_error "Possible timeout in receving first data from donor in gtid stage"
         exit 32
     fi
 
@@ -712,11 +703,9 @@ if ${INNOBACKUPEX_BIN} /tmp --help 2>/dev/null | grep -q -- '--version-check'; t
     disver="--no-version-check"
 fi
 
-iopts+=" --databases-exclude=\"lost+found\""
-
 if [[ ${FORCE_FTWRL:-0} -eq 1 ]];then 
     wsrep_log_info "Forcing FTWRL due to environment variable FORCE_FTWRL equal to $FORCE_FTWRL"
-    iopts+=" --no-backup-locks"
+    iopts+=" --no-backup-locks "
 fi
 
 INNOEXTRA=$WSREP_SST_OPT_MYSQLD
@@ -726,8 +715,7 @@ INNODB_DATA_HOME_DIR=${INNODB_DATA_HOME_DIR:-""}
 if [ ! -z "$INNODB_DATA_HOME_DIR_ARG" ]; then
     INNODB_DATA_HOME_DIR=$INNODB_DATA_HOME_DIR_ARG
 fi
-# if no command line arg and INNODB_DATA_HOME_DIR environment variable
-# is not set, try to get it from my.cnf:
+# if INNODB_DATA_HOME_DIR env. variable is not set, try to get it from my.cnf
 if [ -z "$INNODB_DATA_HOME_DIR" ]; then
     INNODB_DATA_HOME_DIR=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-data-home-dir '')
 fi
@@ -766,9 +754,9 @@ if [[ $ssyslog -eq 1 ]];then
             logger  -p daemon.info -t ${ssystag}wsrep-sst-$WSREP_SST_OPT_ROLE "$@" 
         }
 
-        INNOAPPLY="${INNOBACKUPEX_BIN} --prepare $disver $iapts \$INNOEXTRA $rebuildcmd --target-dir=\${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-apply"
-        INNOMOVE="${INNOBACKUPEX_BIN} ${WSREP_SST_OPT_CONF} --move-back $disver $impts --force-non-empty-directories --target-dir=\${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move"
-        INNOBACKUP="${INNOBACKUPEX_BIN} ${WSREP_SST_OPT_CONF} --backup $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt --target-dir=\$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
+        INNOAPPLY="${INNOBACKUPEX_BIN} --innobackupex $disver $iapts \$INNOEXTRA --apply-log \$rebuildcmd \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-apply"
+        INNOMOVE="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} 2>&1 | logger -p daemon.err -t ${ssystag}innobackupex-move"
+        INNOBACKUP="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2> >(logger -p daemon.err -t ${ssystag}innobackupex-backup)"
     fi
 
 else
@@ -830,9 +818,9 @@ then
 
 fi
  
-    INNOAPPLY="${INNOBACKUPEX_BIN} --prepare $disver $iapts \$INNOEXTRA $rebuildcmd --target-dir=\${DATA} &> ${INNOAPPLYLOG}"
-    INNOMOVE="${INNOBACKUPEX_BIN} ${WSREP_SST_OPT_CONF} --move-back $disver $impts --force-non-empty-directories --target-dir=\${DATA} &> ${INNOMOVELOG}"
-    INNOBACKUP="${INNOBACKUPEX_BIN} ${WSREP_SST_OPT_CONF} --backup $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt --target-dir=\$itmpdir 2> ${INNOBACKUPLOG}"
+    INNOAPPLY="${INNOBACKUPEX_BIN} --innobackupex $disver $iapts \$INNOEXTRA --apply-log \$rebuildcmd \${DATA} &> ${INNOAPPLYLOG}"
+    INNOMOVE="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $impts  --move-back --force-non-empty-directories \${DATA} &> ${INNOMOVELOG}"
+    INNOBACKUP="${INNOBACKUPEX_BIN} --innobackupex ${WSREP_SST_OPT_CONF} $disver $iopts \$tmpopts \$INNOEXTRA --galera-info --stream=\$sfmt \$itmpdir 2> ${INNOBACKUPLOG}"
 fi
 
 get_stream
@@ -855,7 +843,7 @@ then
               -z $(parse_cnf --mysqld tmpdir "") && \
               -z $(parse_cnf xtrabackup tmpdir "") ]]; then
             xtmpdir=$(mktemp -d)
-            tmpopts="--tmpdir=$xtmpdir"
+            tmpopts=" --tmpdir=$xtmpdir"
             wsrep_log_info "Using $xtmpdir as xtrabackup temporary directory"
         fi
 
@@ -868,10 +856,10 @@ then
         fi
 
         if [ -n "${WSREP_SST_OPT_PSWD:-}" ]; then
-            export MYSQL_PWD=$WSREP_SST_OPT_PSWD
+           INNOEXTRA+=" --password=$WSREP_SST_OPT_PSWD"
         elif [[ $usrst -eq 1 ]];then
-            # Empty password, used for testing, debugging etc.
-            unset MYSQL_PWD
+           # Empty password, used for testing, debugging etc.
+           INNOEXTRA+=" --password="
         fi
 
         get_keys
@@ -975,25 +963,17 @@ then
 
     ib_home_dir=$INNODB_DATA_HOME_DIR
 
-    WSREP_LOG_DIR=${WSREP_LOG_DIR:-""}
-    # Try to set WSREP_LOG_DIR from the command line:
-    if [ ! -z "$INNODB_LOG_GROUP_HOME_ARG" ]; then
-        WSREP_LOG_DIR=$INNODB_LOG_GROUP_HOME_ARG
+    # Try to set ib_log_dir from the command line:
+    ib_log_dir=$INNODB_LOG_GROUP_HOME_ARG
+    if [ -z "$ib_log_dir" ]; then
+        ib_log_dir=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-log-group-home-dir "")
     fi
-    # if no command line arg and WSREP_LOG_DIR is not set,
-    # try to get it from my.cnf:
-    if [ -z "$WSREP_LOG_DIR" ]; then
-        WSREP_LOG_DIR=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-log-group-home-dir '')
+    if [ -z "$ib_log_dir" ]; then
+        ib_log_dir=$(parse_cnf --mysqld innodb-log-group-home-dir "")
     fi
-    if [ -z "$WSREP_LOG_DIR" ]; then
-        WSREP_LOG_DIR=$(parse_cnf --mysqld innodb-log-group-home-dir '')
-    fi
-
-    ib_log_dir=$WSREP_LOG_DIR
 
     # Try to set ib_undo_dir from the command line:
-    ib_undo_dir=${INNODB_UNDO_DIR_ARG:-""}
-    # if no command line arg then try to get it from my.cnf:
+    ib_undo_dir=$INNODB_UNDO_DIR_ARG
     if [ -z "$ib_undo_dir" ]; then
         ib_undo_dir=$(parse_cnf mysqld$WSREP_SST_OPT_SUFFIX_VALUE innodb-undo-directory "")
     fi
@@ -1127,7 +1107,7 @@ then
 
             wsrep_log_info "Compressed qpress files found"
 
-	    if ! command -v qpress >/dev/null;then
+            if ! command -v qpress >/dev/null;then
                 wsrep_log_error "qpress not found in path: $PATH"
                 exit 22
             fi
