@@ -6,19 +6,19 @@
 /*  This file contains the JDBC connection classes functions.          */
 /***********************************************************************/
 
-#if defined(_WIN32)
+#if defined(__WIN__)
 // This is needed for RegGetValue
 #define _WINVER 0x0601
 #undef  _WIN32_WINNT
 #define _WIN32_WINNT 0x0601
-#endif   // _WIN32
+#endif   // __WIN__
 
 /***********************************************************************/
 /*  Include relevant MariaDB header file.                              */
 /***********************************************************************/
 #include <my_global.h>
 #include <m_string.h>
-#if defined(_WIN32)
+#if defined(__WIN__)
 //nclude <io.h>
 //nclude <fcntl.h>
 #include <direct.h>                      // for getcwd
@@ -26,7 +26,7 @@
 #define __MFC_COMPAT__                   // To define min/max as macro
 #endif   // __BORLANDC__
 //#include <windows.h>
-#else   // !_WIN32
+#else   // !__WIN__
 #if defined(UNIX)
 #include <errno.h>
 #else   // !UNIX
@@ -36,7 +36,7 @@
 #include <stdlib.h>                      // for getenv
 //nclude <fcntl.h>
 #define NODW
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 /***********************************************************************/
 /*  Required objects includes.                                         */
@@ -53,9 +53,9 @@
 #include "osutil.h"
 
 
-//#if defined(_WIN32)
+//#if defined(__WIN__)
 //extern "C" HINSTANCE s_hModule;           // Saved module handle
-//#endif   // _WIN32
+//#endif   // __WIN__
 #define nullptr 0
 
 TYPCONV GetTypeConv();
@@ -766,7 +766,6 @@ void JDBConn::AddJars(PSTRG jpop, char sep)
 /***********************************************************************/
 bool JDBConn::Connect(PJPARM sop)
 {
-	int      irc = RC_FX;
 	bool		 err = false;
 	jint     rc;
 	jboolean jt = (trace(1));
