@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2013, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2019, MariaDB Corporation.
+Copyright (c) 2018, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -33,27 +33,27 @@ Created 2013-03-16 Sunny Bains
 
 
 /** Class that manages dynamic buffers. It uses a UT_LIST of
-dyn_buf_t::block_t instances. We don't use STL containers in
+mtr_buf_t::block_t instances. We don't use STL containers in
 order to avoid the overhead of heap calls. Using a custom memory
 allocator doesn't solve the problem either because we have to get
 the memory from somewhere. We can't use the block_t::m_data as the
 backend for the custom allocator because we would like the data in
 the blocks to be contiguous. */
-template <size_t SIZE = DYN_ARRAY_DATA_SIZE>
-class dyn_buf_t {
+class mtr_buf_t {
 public:
+	/** SIZE - sizeof(m_node) + sizeof(m_used) */
+	enum { MAX_DATA_SIZE = DYN_ARRAY_DATA_SIZE
+	       - sizeof(ilist_node<>) + sizeof(uint32_t) };
 
 	class block_t : public ilist_node<> {
 	public:
 
 		block_t()
 		{
-			ut_ad(MAX_DATA_SIZE <= (2 << 15));
+			compile_time_assert(MAX_DATA_SIZE <= (2 << 15));
 			init();
 		}
 
-		~block_t() { }
-
 		/**
 		Gets the number of used bytes in a block.
 		@return	number of bytes used */
@@ -108,12 +108,12 @@ public:
 		/**
 		@return pointer to start of reserved space */
 		template <typename Type>
-		Type push(ib_uint32_t size)
+		Type push(uint32_t size)
 		{
 			Type	ptr = reinterpret_cast<Type>(end());
 
 			m_used += size;
-			ut_ad(m_used <= static_cast<ib_uint32_t>(MAX_DATA_SIZE));
+			ut_ad(m_used <= uint32_t(MAX_DATA_SIZE));
 
 			return(ptr);
 		}
@@ -127,7 +127,7 @@ public:
 			ut_ad(ptr <= begin() + m_buf_end);
 
 			/* We have done the boundary check above */
-			m_used = static_cast<ib_uint32_t>(ptr - begin());
+			m_used = uint32_t(ptr - begin());
 
 			ut_ad(m_used <= MAX_DATA_SIZE);
 			ut_d(m_buf_end = 0);
@@ -150,29 +150,20 @@ public:
 		ulint		m_magic_n;
 #endif /* UNIV_DEBUG */
 
-		/** SIZE - sizeof(m_node) + sizeof(m_used) */
-		enum {
-			MAX_DATA_SIZE = SIZE
-				      - sizeof(ilist_node<>)
-				      + sizeof(ib_uint32_t)
-		};
-
 		/** Storage */
 		byte		m_data[MAX_DATA_SIZE];
 
 		/** number of data bytes used in this block;
 		DYN_BLOCK_FULL_FLAG is set when the block becomes full */
-		ib_uint32_t	m_used;
+		uint32_t	m_used;
 
-		friend class dyn_buf_t;
+		friend class mtr_buf_t;
 	};
 
 	typedef sized_ilist<block_t> list_t;
 
-	enum { MAX_DATA_SIZE = block_t::MAX_DATA_SIZE};
-
 	/** Default constructor */
-	dyn_buf_t()
+	mtr_buf_t()
 		:
 		m_heap(),
 		m_size()
@@ -181,7 +172,7 @@ public:
 	}
 
 	/** Destructor */
-	~dyn_buf_t()
+	~mtr_buf_t()
 	{
 		erase();
 	}
@@ -246,7 +237,7 @@ public:
 	@param size	in bytes of the element
 	@return	pointer to the element */
 	template <typename Type>
-	Type push(ib_uint32_t size)
+	Type push(uint32_t size)
 	{
 		ut_ad(size > 0);
 		ut_ad(size <= MAX_DATA_SIZE);
@@ -266,17 +257,11 @@ public:
 	Pushes n bytes.
 	@param str	string to write
 	@param len	string length */
-	void push(const byte* ptr, ib_uint32_t len)
+	void push(const byte* ptr, uint32_t len)
 	{
 		while (len > 0) {
-			ib_uint32_t	n_copied;
-
-			if (len >= MAX_DATA_SIZE) {
-				n_copied = MAX_DATA_SIZE;
-			} else {
-				n_copied = len;
-			}
-
+			uint32_t n_copied = std::min(len,
+						     uint32_t(MAX_DATA_SIZE));
 			::memmove(push<byte*>(n_copied), ptr, n_copied);
 
 			ptr += n_copied;
@@ -292,7 +277,7 @@ public:
 	const Type at(ulint pos) const
 	{
 		block_t*	block = const_cast<block_t*>(
-			const_cast<dyn_buf_t*>(this)->find(pos));
+			const_cast<mtr_buf_t*>(this)->find(pos));
 
 		return(reinterpret_cast<Type>(block->begin() + pos));
 	}
@@ -318,8 +303,7 @@ public:
 #ifdef UNIV_DEBUG
 		ulint	total_size = 0;
 
-		for (typename list_t::iterator it = m_list.begin(),
-					       end = m_list.end();
+		for (list_t::iterator it = m_list.begin(), end = m_list.end();
 		     it != end; ++it) {
 			total_size += it->used();
 		}
@@ -335,8 +319,7 @@ public:
 	template <typename Functor>
 	bool for_each_block(Functor& functor) const
 	{
-		for (typename list_t::iterator it = m_list.begin(),
-					       end = m_list.end();
+		for (list_t::iterator it = m_list.begin(), end = m_list.end();
 		     it != end; ++it) {
 
 			if (!functor(&*it)) {
@@ -353,8 +336,8 @@ public:
 	template <typename Functor>
 	bool for_each_block_in_reverse(Functor& functor) const
 	{
-		for (typename list_t::reverse_iterator it = m_list.rbegin(),
-						       end = m_list.rend();
+		for (list_t::reverse_iterator it = m_list.rbegin(),
+					      end = m_list.rend();
 		     it != end; ++it) {
 
 			if (!functor(&*it)) {
@@ -371,8 +354,8 @@ public:
 	template <typename Functor>
 	bool for_each_block_in_reverse(const Functor& functor) const
 	{
-		for (typename list_t::reverse_iterator it = m_list.rbegin(),
-						       end = m_list.rend();
+		for (list_t::reverse_iterator it = m_list.rbegin(),
+					      end = m_list.rend();
 		     it != end; ++it) {
 
 			if (!functor(&*it)) {
@@ -401,8 +384,8 @@ public:
 
 private:
 	// Disable copying
-	dyn_buf_t(const dyn_buf_t&);
-	dyn_buf_t& operator=(const dyn_buf_t&);
+	mtr_buf_t(const mtr_buf_t&);
+	mtr_buf_t& operator=(const mtr_buf_t&);
 
 	/**
 	Add the block to the end of the list*/
@@ -413,9 +396,9 @@ private:
 	}
 
 	/** @return the last block in the list */
-	block_t* back()
+	block_t* back() const
 	{
-		return &m_list.back();
+		return &const_cast<block_t&>(m_list.back());
 	}
 
 	/*
@@ -440,8 +423,7 @@ private:
 	{
 		ut_ad(!m_list.empty());
 
-		for (typename list_t::iterator it = m_list.begin(),
-					       end = m_list.end();
+		for (list_t::iterator it = m_list.begin(), end = m_list.end();
 		     it != end; ++it) {
 
 			if (pos < it->used()) {
@@ -490,8 +472,6 @@ private:
 	block_t			m_first_block;
 };
 
-typedef dyn_buf_t<DYN_ARRAY_DATA_SIZE> mtr_buf_t;
-
 /** mtr_buf_t copier */
 struct mtr_buf_copy_t {
 	/** The copied buffer */
